
# q-expansion of newform 549.2.s.j, downloaded from the LMFDB on 18 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 549
weight = 2
poly_data = [16, -8, 8, -8, 1, -4, 2, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0, 0, 0, 0], 1], [[-8, 8, 6, 7, 0, 0, -1, -1], 8], [[-8, 8, -8, 1, -4, 2, -1, 1], 8], [[-16, -2, -7, 1, 0, 3, 0, 1], 4], [[-16, -8, -10, 1, 0, 4, 1, 1], 4], [[-32, -4, -20, 3, 0, 6, 1, 3], 8], [[-8, 1, -5, 1, -1, 2, 0, 1], 2]]

hecke_ring_character_values = [[245, [1, 0, 0, 0, 0, 0, 0, 0]], [307, [1, 0, 0, 0, 1, 0, 0, 0]]]
aps_data = [[0, -1, 0, 0, 0, 0, -1, 0], [0, 0, 0, 0, 0, 0, 0, 0], [0, -1, 0, 0, 1, 0, 1, 0], [0, 0, 0, 0, 0, 0, 0, -1], [-1, 0, 0, 0, -2, 1, 1, 0], [0, -1, 0, 1, 0, 1, 2, 0], [1, -2, 2, 1, 1, -1, 0, 0], [2, -1, 1, -1, 1, 0, 0, -2], [-1, -2, 1, -1, -4, -1, 0, -1], [0, 2, 1, -1, 1, 1, -3, 0], [1, -1, 2, -2, 1, -3, -1, 0], [1, -1, 2, 1, -2, -3, -1, -2], [2, 1, 0, -1, 0, 3, -3, 0], [-2, -2, 0, -2, -1, 2, -4, 2], [-1, -2, 1, 0, -2, -2, -1, -2], [-4, 1, -4, -3, 0, 2, -2, 4], [4, 2, 0, 3, 2, -3, 5, -4], [-5, 1, -1, 1, -6, 4, -1, 0], [0, 1, 0, 1, 0, -1, 2, -4], [-3, 3, -2, -4, 1, -1, -1, 0], [-8, 2, -1, -4, -7, 6, 1, 2], [8, 0, 0, 3, 4, -3, 3, 1], [0, 1, 0, 0, -10, 0, -1, 0], [7, -2, 4, 2, 6, -4, 0, -4], [6, -1, 2, 4, 4, -5, 1, -4], [-4, 0, 0, -2, -2, 2, -2, -2], [-1, -2, 0, 6, -1, -8, -2, 0], [-8, 4, -8, 0, -8, 8, 4, 4], [2, -2, 2, 3, 4, 1, 3, -1], [-1, 2, 1, -3, 0, -2, 1, 1], [0, -2, 0, -1, -3, -1, 1, 0], [7, 2, -1, -1, 0, -2, 3, -1], [-2, -1, -2, -4, -5, -2, -1, 1], [2, -11, 1, 8, -1, -3, 10, 0], [2, -3, 1, 2, 0, -3, 2, 1], [2, 4, -3, -5, -5, -1, -1, 0], [1, 6, -6, -9, -7, -3, 0, 0], [-5, 7, -3, -4, 0, 4, -1, -3], [2, -1, 2, -4, 3, -6, -5, -1], [4, -9, 6, 7, 2, -2, 3, 0], [3, 4, 0, -4, 3, 8, 4, 0], [-6, -4, 0, -1, -3, 1, -5, 2], [15, -6, 7, 1, 16, -10, -3, -7], [-5, 4, 1, -1, 6, 3, -5, 0], [0, -5, 1, -1, -1, -4, -4, -2], [8, -5, 8, 3, 2, -5, 0, -4], [-7, -1, -6, -7, -2, 1, -5, 6], [2, 10, 0, -5, 1, 5, 5, -4], [3, 2, 0, 7, -3, 9, -2, 0], [-4, -3, -4, -7, 5, -3, 0, 2], [-1, 1, -4, -3, 6, 6, 2, 4], [-2, -5, -2, -1, -4, 1, 6, 1], [6, -8, -1, 9, 0, 0, 1, -1], [2, -4, 0, 6, 1, -6, 2, 0], [-6, 7, -6, -1, 6, 5, -2, 3], [-12, 3, 0, 0, -12, 3, 3, 0], [8, -3, 4, 4, 4, -7, 1, -8], [-7, -2, -2, 4, 0, -6, 8, -2], [-7, 14, -6, 8, -2, 13, 7, 6], [6, -4, 3, -1, 6, -7, -4, -3], [-4, 0, -4, -2, -10, 2, 2, 2], [-13, 3, 0, -5, -13, 8, 3, 0], [-4, 0, 0, -9, -2, 9, -9, 4], [-6, -4, 0, -6, -3, 6, -10, 0], [11, -8, 5, -3, 12, -8, -3, -5], [8, -7, 4, 5, 4, -12, -3, -8], [4, 9, -3, -3, -7, 6, -6, 0], [9, -5, 6, -2, -3, -7, -1, 0], [10, 10, 3, 10, 7, 0, 13, -6], [-8, 11, -3, -7, 5, 4, -8, 0], [-6, -5, -6, -7, -9, -1, 4, 3], [4, 2, 0, 3, 2, -3, 5, 0], [-16, -4, 4, 0, 0, -2, -2, 4], [2, -11, 0, 1, 1, -1, -10, -1], [0, 5, 0, 10, -1, 10, 5, 0], [2, -13, 4, 7, 2, -6, 9, 0], [-2, 2, -5, -3, 6, -9, -14, 5], [-2, -1, -2, -3, 0, -4, 3, 0], [-7, 13, -5, -8, 2, 5, -8, 0], [6, 2, 0, 2, 3, -2, 4, -8], [0, -3, -2, -5, 4, -16, -18, 2], [16, -5, 0, 4, 8, -4, -1, 1], [-8, 2, -1, 0, -7, 2, 1, 2], [0, -12, 0, 9, 0, -9, -3, 6], [-6, 0, 2, 10, -8, -10, 2, -4], [0, 1, 0, 7, -23, 7, 6, 0], [4, -5, 4, 10, -12, 6, 11, -2], [5, 4, -4, -3, -9, 1, 0, 0], [8, -6, -3, 9, 0, 7, -4, -3], [-17, -8, -3, -11, -14, 3, -11, 6], [14, 1, 0, 3, 7, -3, 4, 8], [-2, -1, 7, -6, 0, 5, -12, 7], [-19, 0, -2, 2, 0, -2, 4, -2], [7, -10, 7, 3, 0, -7, 0, 7], [2, -6, 1, -5, 2, -8, -7, -1], [-12, 3, -12, -12, -12, 0, -3, 6], [20, -11, 0, 9, 10, -9, -2, -10], [17, 7, 2, 9, 30, 5, 7, -2], [14, -11, 8, 0, -6, -11, 3, 0], [-8, -2, -2, -4, -12, -9, -11, 2], [10, 0, 0, 3, 5, -3, 3, -13], [4, 3, 3, 6, 2, -1, 2, -3], [-2, -3, 3, 8, -5, -11, 0, -6], [6, 6, -6, 0, 0, 3, 3, -6], [2, -7, 2, 2, -9, 0, 7, -1], [-7, 14, -2, 12, -10, 8, 6, 2], [-16, 10, -7, 3, -18, 18, 11, 7], [-18, 10, -11, 1, 7, 11, 1, 0], [10, 0, 3, 3, 14, 1, 4, -3], [-22, 8, -4, -4, 0, 12, -8, -4], [-4, 8, 4, -12, 0, -1, -3, 4], [20, -15, -3, 18, 0, -5, 8, -3], [-16, -3, 0, -8, -8, 8, -11, -2], [-2, -4, -2, 6, 0, -4, 6, -2], [-2, -4, 0, -10, -1, 10, -14, 0], [10, -15, 9, 8, -1, -7, 6, 0], [8, -1, 8, 7, 0, -9, -1, -8], [-5, 5, 0, 5, -10, 11, 11, 0], [14, 4, 0, -7, 7, 7, -3, 7], [-20, -8, -1, -5, -19, -3, -9, 2], [1, 13, 2, -9, 1, 4, -15, 0], [22, -12, 0, 3, 11, -3, -9, -3], [-2, -9, 0, 10, -1, -10, 1, 4], [16, -1, 6, 6, 10, -7, 5, -12], [14, -18, 4, 14, 0, -9, 5, 4], [-12, 13, -1, -17, 11, -4, -12, 0], [31, -8, 9, 1, 44, -13, -4, -9], [-12, 12, -12, -4, 13, 8, -4, 6], [2, 5, 2, -3, 5, -5, -10, -1], [-15, 8, 4, -12, 0, 5, -9, 4], [-14, 0, 0, 9, -7, -9, 9, -2], [4, -13, 8, -5, -8, -10, -2, -8], [-23, 5, -10, -13, -13, 18, -5, 20], [32, 11, 7, 6, 25, 5, 18, -14], [10, 3, -6, -5, -16, -2, 3, 0], [1, -11, 3, -8, -4, -1, 2, -3], [3, -5, 4, 1, 0, 4, -8, 4], [-2, 8, 0, -4, -1, 4, 4, -1], [-28, 16, -2, -6, -26, 22, 14, 4], [-18, 15, -18, -8, -1, 10, -5, 9], [52, -5, 0, 4, 26, -4, -1, -5], [5, 2, 2, 4, 6, 10, 12, -2], [6, -9, -2, -8, -8, -17, 11, 0], [-3, 4, 6, -10, 0, -4, -2, 6], [-24, 11, -2, 0, -22, 11, 9, 4], [10, -4, 10, -6, 15, -16, -12, -5], [17, 2, 6, -8, 0, 13, -19, 6], [-19, -4, 7, -3, 0, -5, -2, 7], [4, 0, 4, 17, 17, 13, 13, -2], [-8, 3, 8, -11, 0, 6, -14, 8], [-4, 7, -2, 5, -4, 15, 13, 2], [14, 7, 0, -7, 0, 0, 0, 0], [7, -9, 3, -3, -4, -12, 6, 0], [32, -13, 16, 3, 32, -18, -2, -16], [-3, -6, 7, 1, -20, 13, 20, -7], [2, -4, -2, 11, 4, -15, -6, 4], [-24, 1, -2, -8, -22, 9, -1, 4], [-11, 17, 1, 9, -12, 8, 18, -2], [-22, 16, -22, -13, 1, 9, -7, 11], [-9, 6, -13, -6, -4, 0, 7, 0], [-7, -15, -9, 14, -2, -1, 24, 0], [-26, 0, -12, -12, -28, 1, -11, 12], [-24, 2, -2, 0, 0, -1, 3, -2], [8, 11, 8, -2, -2, -10, -21, -4], [9, -11, 4, 7, 0, -11, 7, 4], [-16, -14, 0, 9, -8, -9, -5, 3], [2, 5, 2, -16, 12, -18, -23, -1], [8, 0, 0, 9, 4, -9, 9, -4]]
